#
# Copyright (c) 2011 CNRS/LAAS
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
package provide Gdhe::models 1.0
# Neobotix MOR platform

objectDelete morwheel
objectDelete morbase
objectDelete morhokuyo

# roue
proc morwheel { theta } {
    pushMatrix
    rotate $theta 0 0 1
    object morwheel {
	color 120 110 60
	cylinder 0 -0.04 0.075 y 0.15 0.04
	color 160 160 160
	cylinder 0 -0.06 0.075 y 0.075 0.02
	box 0 -0.05 0.075 0.075 0.02 0.1
	cylinder 0 0 0.075 y 0.03 0.04
	cylinder 0 0.04 0.075 y 0.075 0.02
	box 0 0.05 0.075 0.075 0.02 0.1
	cylinder 0 0 0.16 z 0.15 0.04
    }
    popMatrix
}

set wx(0) 0.24
set wy(0) 0.20
set wt(0) -180
set wx(1) -0.24
set wy(1) 0.20
set wt(1) -180
set wx(2) -0.24
set wy(2) -0.20
set wt(2) 0
set wx(3) 0.24
set wy(3) -0.20
set wt(3) 0

proc morwheels { t1 t2 t3 t4 } {
    global wx wy wt

    for {set i 0} {$i < 4} {incr i} {
	pushMatrix
	translate $wx($i) $wy($i) 0 
	morwheel [expr $wt($i) + [set t[expr $i + 1]]]
	popMatrix
    }
}

proc morhokuyo {} {
    object morhokuyo {
	color 160 160 160 
	box 0 0 0 0.08 0.10 0.01 
	color 40 40 40 
	box 0 0 0.01 0.06 0.06 0.02
	box 0 0 0.03 0.055 0.055 0.01
	color 40 40 40
	cylinder 0 0 0.03 z 0.05 0.04
	color 200 100 0
	box 0 0 0.07 0.055 0.055 0.01

	color 40 40 40
	box 0 0 0.08 0.055 0.055 0.005
    }
}

proc mor {{t1 0} {t2 0} {t3 0} {t4 0}} {
    morwheels $t1 $t2 $t3 $t4
    object morbase {
	color 160 160 160
	box 0 0 0.2 0.6 0.51 0.01
	cylinder 0.24 0.20 0.21 z 0.12 0.09
	cylinder 0.24 -0.20 0.21 z 0.12 0.09
	cylinder -0.24 0.20 0.21 z 0.12 0.09
	cylinder -0.24 -0.20 0.21 z 0.12 0.09
	box 0.24 0.14 0.21 0.12 0.12 0.09
	box 0.24 -0.14 0.21 0.12 0.12 0.09
	box -0.24 0.14 0.21 0.12 0.12 0.09
	box -0.24 -0.14 0.21 0.12 0.12 0.09
	box 0 0 0.3 0.6 0.51 0.04
	box 0.27 0.225 0.34 0.06 0.06 0.17
	box 0.27 -0.225 0.34 0.06 0.06 0.17
	box -0.27 0.225 0.34 0.06 0.06 0.17
	box -0.27 -0.225 0.34 0.06 0.06 0.17
	box 0 0 0.51 0.6 0.51 0.04
	color 80 80 80
	# internal boxes
	box -0.12 0 0.21 0.30 0.20 0.1
	box 0.10 0 0.34 0.20 0.40 0.18
	# E-Stop
	color 200 180 0
	cylinder 0.26 0 0.55 z 0.04 0.02
	color 160 0 0
	cylinder 0.26 0 0.57 z 0.025 0.012
	cylinder 0.26 0 0.582 z 0.03 0.002
    }
    pushMatrix
    translate 0.32 0.27 0.2
    rotate 45 0 0 1
    morhokuyo
    popMatrix
    pushMatrix
    translate -0.32 -0.27 0.2
    rotate 45 0 0 1
    morhokuyo
    popMatrix
}


proc morotate {r} {
    global wx wy wt
    global robots

    for {set i 0} {$i < 4} {incr i} {
	set rw [expr $r - $wy($i)]
#	if {abs($rw) < 0.003} {
#	    if {$wx($i) > 0} {
#		set theta($i) -90
#	    } else {
#		set theta($i) 90
#	    }
#	} else {
	set theta($i) [rad2deg atan2($wx($i),$rw)]
#	if { $theta($i) > 90 }  {
#	    set theta($i) [expr  $theta($i) - 180]
#	}
#	if { $theta($i) < -90 }  {
#	    set theta($i) [expr -180 + $theta($i)]
#	}
#	}
    }
    set robots(r) "mor $theta(0) $theta(1) $theta(2) $theta(3)"
    puts $robots(r)
}
